/* This is the library of routines that are used by the contestants to
   simulate dropping the objects.

   A compiled version of this library was given to each contestant.
   The source code was not given to them.
 */

#include <stdio.h>
#include <stdlib.h>

#define MAX_N 100
#define NUM_BALLS 2

static int N, D, H;
static int drops_left, balls_left;
static int init = 0;
static FILE *outlog = 0;



static void 
error_quit (char *msg)
{
  printf ("Level Library: Error: %s\n", msg);
  printf ("The Level Library is terminating your program.\n");
  if (outlog)
    {
      fprintf (outlog, "Level Library: Error: %s\n", msg);
      fprintf (outlog, "The Level Library is terminating your program.\n");
      fflush (outlog);
      fclose (outlog);
    }
  exit (1);
}

static int 
initialize (void)
{
  int result;
  FILE *infp, *outfp;

  if (init)
    return 1;


  outlog = fopen ("level.log", "w");
  if (!outlog)
    {
      error_quit ("Cannot open output file level.log");
    }
  fprintf (outlog, "Starting simulation\n");
  fflush (outlog);

  outfp = fopen ("level.out", "w");
  if (!outfp)
    {
      error_quit ("Cannot open output file level.out");
    }
  fprintf (outfp, "No value was passed to Decide\n");
  fclose (outfp);



  infp = fopen ("level.in", "r");
  if (!infp)
    {
      error_quit ("Unable to open level.in");
    }
  result = fscanf (infp, "%d %d %d", &N, &D, &H);
  fclose (infp);
  if (3 != result)
    {
      error_quit ("Unable to read three integers from level.in");
    }





  if (N < 0 || N > MAX_N)
    {
      error_quit ("Invalid value for N in level.in");
    }
  if (D < 0 || D > N)
    {
      error_quit ("Invalid value for D in level.in");
    }
  if (H < 1 || H > N)
    {
      error_quit ("Invalid value for H in level.in");
    }


  drops_left = D;
  balls_left = NUM_BALLS;
  fprintf (outlog, "Reading parameters from level.in\n\tD=%d, N=%d, H=%d\n", D, N, H);
  fflush (outlog);
  init = 1;
  return 1;
}

int 
GetN (void)
{
  initialize ();
  fprintf (outlog, "GetN returning %d\n", N);
  fflush (outlog);
  return N;
}

int 
GetD (void)
{
  initialize ();
  fprintf (outlog, "GetD returning %d\n", D);
  fflush (outlog);
  return D;
}

int 
Drop (int level)
{
  initialize ();

  fprintf (outlog, "Drop(%d)\n", level);
  fflush (outlog);

  if (balls_left < 1)
    {
      error_quit ("Cannot perform drop: the two samples have already broken");
    }
  if (drops_left < 1)
    {
      error_quit ("Cannot perform drop: drop limit exceeded");
    }
  if (level < 1 || level > N)
    {
      error_quit ("Cannot perform drop: invalid level");
    }


  drops_left--;
  if (level >= H)
    {
      balls_left--;
      fprintf (outlog, "\tDrop returning 1\n");
      fflush (outlog);
      return 1;
    }
  fprintf (outlog, "\tDrop returning 0\n");
  fflush (outlog);
  return 0;
}

void 
Decide (int level)
{
  FILE *outfp;
  initialize ();
  fprintf (outlog, "Decide(%d)\n", level);
  fflush (outlog);
  outfp = fopen ("level.out", "w");
  if (!outfp)
    {
      error_quit ("Cannot open output file level.out");
    }
  fprintf (outfp, "%d\n", level);
  fclose (outfp);

  if (level == H)
    {
      fprintf (outlog, "\tCongratulations! You have found the breaking level.\n");
    }
  else
    {
      fprintf (outlog, "\tYour answer is incorrect.\n\tThe correct answer is %d.\n", H);
    }
  fprintf (outlog, "Ending simulation\n");
  fflush (outlog);
  fclose (outlog);
  exit (0);
  init = 0;
}
